import logging
import os

from telegram.ext import CallbackQueryHandler, CommandHandler, Updater

from .cmds.help import cmd_help
from .cmds.start import cmd_start
from .constant import (
    CBK_DATA_HOME,
    CBK_DATA_LIST,
    CBK_DATA_REFRESH,
    CBK_DATA_REMOVE,
    CBK_DATA_REMOVE_LIST,
    CBK_DATA_RM_N,
    CBK_DATA_RM_Y,
    CBK_DATA_SHOW,
    CMD_HELP,
    CMD_START,
    TOKEN_ENV_VAR,
)
from .handlers.add import cnvs_hdl_add
from .handlers.delete import cbk_rm, cbk_rm_confirm_n, cbk_rm_confirm_y, cbk_rm_ls
from .handlers.list import cbk_ls
from .handlers.show import cbk_refresh, cbk_show
from .handlers.start import cbk_home
from .helpers.env import check_environments

logger = logging.getLogger(__name__)


def main() -> None:

    check_environments()

    """Start the bot."""
    updater = Updater(os.getenv(TOKEN_ENV_VAR))

    # Get the dispatcher to register handlers
    dispatcher = updater.dispatcher

    # -- Commands
    dispatcher.add_handler(CommandHandler(CMD_START, cmd_start))
    dispatcher.add_handler(CommandHandler(CMD_HELP, cmd_help))

    # -- Conversation handlers
    dispatcher.add_handler(cnvs_hdl_add())  # add

    # -- Callback query

    # Back home callback
    dispatcher.add_handler(CallbackQueryHandler(cbk_home, pattern=CBK_DATA_HOME))
    # List applications callback
    dispatcher.add_handler(CallbackQueryHandler(cbk_ls, pattern=CBK_DATA_LIST))

    # Manage remove
    dispatcher.add_handler(
        CallbackQueryHandler(cbk_rm_ls, pattern=CBK_DATA_REMOVE_LIST)
    )
    dispatcher.add_handler(CallbackQueryHandler(cbk_rm, pattern=CBK_DATA_REMOVE))
    dispatcher.add_handler(
        CallbackQueryHandler(cbk_rm_confirm_y, pattern=CBK_DATA_RM_Y)
    )
    dispatcher.add_handler(
        CallbackQueryHandler(cbk_rm_confirm_n, pattern=CBK_DATA_RM_N)
    )

    # Manage show
    dispatcher.add_handler(
        CallbackQueryHandler(cbk_show, pattern=f"{CBK_DATA_SHOW}__.*")
    )
    dispatcher.add_handler(
        CallbackQueryHandler(cbk_refresh, pattern=f"{CBK_DATA_REFRESH}__.*")
    )

    # Start the Bot
    updater.start_polling()

    # Run the bot until you press Ctrl-C or the process receives SIGINT,
    # SIGTERM or SIGABRT. This should be used most of the time, since
    # start_polling() is non-blocking and will stop the bot gracefully.
    updater.idle()



import base64
import os

from dynamodb_encryption_sdk.delegated_keys.jce import JceNameLocalDelegatedKey
from dynamodb_encryption_sdk.identifiers import EncryptionKeyType, KeyEncodingType
from dynamodb_encryption_sdk.material_providers.wrapped import (
    WrappedCryptographicMaterialsProvider,
)

from ..constant import CRYPTO_ENCRYPTION_ENV_VAR, CRYPTO_SIGN_ENV_VAR

_encryption_key = JceNameLocalDelegatedKey(
    key=base64.b64decode(
        os.getenv(CRYPTO_ENCRYPTION_ENV_VAR) or base64.b64encode(os.urandom(32))
    ),
    algorithm="AES",
    key_type=EncryptionKeyType.SYMMETRIC,
    key_encoding=KeyEncodingType.RAW,
)
_signing_key = JceNameLocalDelegatedKey(
    key=base64.b64decode(
        os.getenv(CRYPTO_SIGN_ENV_VAR) or base64.b64encode(os.urandom(32))
    ),
    algorithm="HmacSHA512",
    key_type=EncryptionKeyType.SYMMETRIC,
    key_encoding=KeyEncodingType.RAW,
)

wrapped_material_provider = WrappedCryptographicMaterialsProvider(
    wrapping_key=_encryption_key,
    unwrapping_key=_encryption_key,
    signing_key=_signing_key,
)



import logging

from telegram import Update
from telegram.ext import CallbackContext

from ..helpers.keyboards import default_keyboard

logger = logging.getLogger(__name__)


def cbk_cancel(update: Update, context: CallbackContext) -> None:
    from .add import cnvs_hdl_add

    context.dispatcher.add_handler(cnvs_hdl_add())
    if msg := update.message:
        msg.reply_text(
            "Okay, let's start again..\nChoose somethind to do!",
            reply_markup=default_keyboard(),
        )
    else:
        logger.error("Called cbk_rm_ls with no mesage")



TOKEN_ENV_VAR = "TELEGRAM_API_TOKEN"
DYNAMO_ENDPOINT_ENV_VAR = "DYNAMO_DB_ENDPOINT"
CRYPTO_ENCRYPTION_ENV_VAR = "DYNAMO_AES_ENCRYPTION_KEY"
CRYPTO_SIGN_ENV_VAR = "DYNAMO_AES_SIGN_KEY"

CMD_START = "start"
CMD_HELP = "help"
CMD_ADD = "add"
CMD_CANCEL = "cancel"

BTN_ADD_LABEL = "Add"
BTN_REMOVE_LABEL = "Delete"
BTN_LIST_LABEL = "List"
BTN_REFRESH_LABEL = "Refresh"
BTN_BACK_LABEL = "« Go back"

BTN_REMOVE_Y_LABEL = "Yep"
BTN_REMOVE_N_LABEL = "Nope"


CBK_DATA_ADD = "cbk_add"
CBK_DATA_REMOVE = "cbk_rm"
CBK_DATA_REMOVE_LIST = "cbk_rm_ls"
CBK_DATA_LIST = "cbk_ls"
CBK_DATA_SHOW = "cbk_show"
CBK_DATA_REFRESH = "cbk_refresh"
CBK_DATA_HOME = "cbk_home"
CBK_DATA_RM_PREFIX = "cbk_cnfrm"
CBK_DATA_RM_Y = f"{CBK_DATA_RM_PREFIX}_y"
CBK_DATA_RM_N = f"{CBK_DATA_RM_PREFIX}_n"

LIST_ROW_LENGTH = 3

# States for conversation handler
ST_ADD_NAME = 0
ST_ADD_NAME_ERR = 1
ST_ADD_OPT = 2
ST_ADD_BROKEN = 3
ST_RM_NAME = 4
ST_RM_CONFIRM = 5
ST_RM_CONFIRM_Y = 6
ST_RM_CONFIRM_N = 7

USER_DATA_KEY_ADD_NAME = "add-opt-name-key"
USER_DATA_KEY_ADD_MSG_ID = "add-msg-id-key"
USER_DATA_KEY_DEL_NAME = "del-msg-id-key"

DYNAMO_TABLE_NAME = "yaotpbot-table"
DYNAMO__KEY_ID = "id"
DYNAMO__KEY_NAME = "name"
DYNAMO__KEY_SEED = "seed"



from telegram import Update
from telegram.ext import CallbackContext

from ..helpers.keyboards import default_keyboard


def cbk_home(update: Update, context: CallbackContext) -> None:
    if cbk := update.callback_query:
        context.bot.answer_callback_query(cbk.id)
        if update.message:
            update.message.reply_text(
                "Choose somethind to do!", reply_markup=default_keyboard()
            )
        elif msg := cbk.message:
            context.bot.edit_message_text(
                chat_id=msg.chat.id,
                message_id=msg.message_id,
                text="Choose somethind to do!",
                reply_markup=default_keyboard(),
            )



import base64
import logging
import os

import boto3

from ..constant import CRYPTO_ENCRYPTION_ENV_VAR, CRYPTO_SIGN_ENV_VAR, TOKEN_ENV_VAR

logger = logging.getLogger(__name__)


def check_environments():
    _check_token()
    _check_credentials()
    _check_keys()


def _check_token():
    if os.getenv(TOKEN_ENV_VAR) is None:
        logger.error(
            f"Environment variable {TOKEN_ENV_VAR} must be set with valid Telegram Token."
        )
        exit(-1)


def _check_credentials():
    sts = boto3.client("sts")
    try:
        sts.get_caller_identity()
        logger.info("Valid AWS credentials found.")
    except Exception:
        logger.error("AWS credentials are not set or invalid.")
        exit(-1)


def _check_keys():
    if None in [os.getenv(CRYPTO_ENCRYPTION_ENV_VAR), os.getenv(CRYPTO_SIGN_ENV_VAR)]:
        logger.error(
            f"Environment variables {CRYPTO_ENCRYPTION_ENV_VAR} and {CRYPTO_SIGN_ENV_VAR} must be set."
            "\nYou can use following python script to generate one:\n\nimport os,base64\nbase64.b64encode(os.urandom(16)).decode('utf-8')"
        )
        exit(-1)
    if (
        len(base64.b64decode(os.getenv(CRYPTO_ENCRYPTION_ENV_VAR))) not in [16, 24, 32]
    ) or (len(base64.b64decode(os.getenv(CRYPTO_SIGN_ENV_VAR))) not in [16, 24, 32]):
        logger.error(
            f"{CRYPTO_ENCRYPTION_ENV_VAR} or {CRYPTO_SIGN_ENV_VAR} has invalid length."
            "\nValid length are 128,192 or 256 bits."
            "\nYou can use following python script to generate one:\n\nimport os,base64\nbase64.b64encode(os.urandom(16)).decode('utf-8')"
        )
        exit(-1)



import logging
import os
from typing import List, Tuple

import boto3
from boto3.dynamodb.conditions import Key
from dynamodb_encryption_sdk.encrypted.resource import EncryptedResource

from ..constant import (
    DYNAMO__KEY_ID,
    DYNAMO__KEY_NAME,
    DYNAMO__KEY_SEED,
    DYNAMO_ENDPOINT_ENV_VAR,
    DYNAMO_TABLE_NAME,
)
from ..helpers.crypto import wrapped_material_provider

logger = logging.getLogger(__name__)

dynamo = boto3.resource("dynamodb", endpoint_url=os.getenv(DYNAMO_ENDPOINT_ENV_VAR))
client_crypto = EncryptedResource(
    resource=dynamo, materials_provider=wrapped_material_provider
).Table(DYNAMO_TABLE_NAME)


def get(chat_id: str, otp_name: str) -> str:
    if query := client_crypto.query(
        KeyConditionExpression=Key("id").eq(f"{chat_id}")
        & Key("name").eq(f"{otp_name}"),
    )["Items"]:
        return query[0]["seed"]
    else:
        logger.error(f"Key {otp_name} for {chat_id} was not found.")
        raise Exception(f"Key {otp_name} for {chat_id} was not found.")


def put(chat_id: str, otp_name: str, seed: str):
    client_crypto.put_item(
        Item={
            DYNAMO__KEY_ID: f"{chat_id}",
            DYNAMO__KEY_NAME: f"{otp_name}",
            DYNAMO__KEY_SEED: f"{seed}",
        },
    )


def exists(chat_id: str, otp_name: str) -> bool:
    query = client_crypto.query(
        KeyConditionExpression=Key("id").eq(f"{chat_id}")
        & Key("name").eq(f"{otp_name}"),
    )
    return "Items" in query


def delete(chat_id: str, otp_name: str) -> None:
    client_crypto.delete_item(Key={"id": f"{chat_id}", "name": f"{otp_name}"})


def load(chat_id: str) -> List[Tuple[str, str]]:
    if query := client_crypto.query(
        KeyConditionExpression=Key("id").eq(f"{chat_id}"),
        # ProjectionExpression="#n,seed",
        # ExpressionAttributeNames={"#n": "name"},
    )["Items"]:
        return [(k["name"], k["seed"]) for k in query]
    return []



import logging

from telegram import Update
from telegram.ext import CallbackContext

from ..helpers.keyboards import ls_keyboard

logger = logging.getLogger(__name__)


def cbk_ls(update: Update, context: CallbackContext) -> None:
    if cbk := update.callback_query:
        context.bot.answer_callback_query(cbk.id)
        if msg := cbk.message:
            context.bot.edit_message_text(
                "Select the application to show 👀",
                chat_id=msg.chat.id,
                message_id=msg.message_id,
                reply_markup=ls_keyboard(str(msg.chat.id)),
            )
        else:
            logger.error("Called cbk_rm_ls with no mesage")



import logging

from telegram import Update
from telegram.ext import CallbackContext

from ..constant import USER_DATA_KEY_DEL_NAME
from ..helpers import persistence
from ..helpers.keyboards import default_keyboard, ls_rm_keyboard, rm_keyboard

logger = logging.getLogger(__name__)


def cbk_rm_ls(update: Update, context: CallbackContext) -> None:
    if cbk := update.callback_query:
        context.bot.answer_callback_query(cbk.id)
        if msg := cbk.message:
            context.bot.edit_message_text(
                "Select the application to remove 🗑🗑",
                chat_id=msg.chat.id,
                message_id=msg.message_id,
                reply_markup=ls_rm_keyboard(str(msg.chat.id)),
            )
        else:
            logger.error("Called cbk_rm_ls with no mesage")


def cbk_rm(update: Update, context: CallbackContext) -> None:
    if (
        (cbk := update.callback_query)
        and (data := cbk.data)
        and (userdata := context.user_data)
    ):
        name = "__".join(data.split("__")[1:])
        if msg := cbk.message:
            if persistence.exists(str(msg.chat_id), name):
                userdata[USER_DATA_KEY_DEL_NAME] = name
                context.bot.edit_message_text(
                    f"🥺🥺 Are you sure you want to delete {name}?",
                    chat_id=msg.chat.id,
                    message_id=msg.message_id,
                    reply_markup=rm_keyboard(),
                )

            else:
                context.bot.edit_message_text(
                    f"Oooops, seems like '{name}' isn't your OTPs 🤷🏻‍♂️\nWhat now?",
                    chat_id=msg.chat.id,
                    message_id=msg.message_id,
                    reply_markup=default_keyboard(),
                )
    else:
        logger.error(
            "Called cbk_on_otp with no message or no text on message or no user data."
        )


def cbk_rm_confirm_y(update: Update, context: CallbackContext) -> None:
    if (cbk := update.callback_query) and (userdata := context.user_data):
        context.bot.answer_callback_query(cbk.id)
        otp_name = userdata[USER_DATA_KEY_DEL_NAME]
        if msg := cbk.message:
            persistence.delete(str(msg.chat.id), otp_name)
            context.bot.edit_message_text(
                chat_id=msg.chat.id,
                message_id=msg.message_id,
                text=f"🥳🥳 OTP {otp_name} successfully deleted!\nWhat now?",
                reply_markup=default_keyboard(),
            )
        else:
            logger.error("Called cbk_rm_ls with no mesage")


def cbk_rm_confirm_n(update: Update, context: CallbackContext) -> None:
    if cbk := update.callback_query:
        context.bot.answer_callback_query(cbk.id)
        if msg := cbk.message:
            context.bot.edit_message_text(
                chat_id=msg.chat.id,
                message_id=msg.message_id,
                text="✌️✌️ Nevermind, let's start again. \nWhat to do?",
                reply_markup=default_keyboard(),
            )
        else:
            logger.error("Called cbk_rm_ls with no mesage")



import logging

import pyotp
from telegram import Update
from telegram.ext import CallbackContext

from ..helpers import persistence
from ..helpers.keyboards import show_keyboard

logger = logging.getLogger(__name__)


def cbk_show(update: Update, context: CallbackContext) -> None:
    if (cbk := update.callback_query) and (data := cbk.data):
        context.bot.answer_callback_query(cbk.id)
        name = "__".join(data.split("__")[1:])
        if msg := cbk.message:
            seed = persistence.get(str(msg.chat.id), name)
            otp = pyotp.TOTP(seed)
            context.bot.edit_message_text(
                f"OTP for {name}: {otp.now()}",
                chat_id=msg.chat.id,
                message_id=msg.message_id,
                reply_markup=show_keyboard(name),
            )
        else:
            logger.error("Called cbk_show with no message")


def cbk_refresh(update: Update, context: CallbackContext) -> None:
    if (cbk := update.callback_query) and (data := cbk.data):
        context.bot.answer_callback_query(cbk.id)
        name = "__".join(data.split("__")[1:])
        if msg := cbk.message:
            seed = persistence.get(str(msg.chat.id), name)
            otp = pyotp.TOTP(seed)
            if raw_old_otp := msg.text:
                oldotp = raw_old_otp.split(" ")[-1]
                newotp = otp.now()
                if oldotp != newotp:
                    context.bot.edit_message_text(
                        f"OTP for {name}: {newotp}",
                        chat_id=msg.chat.id,
                        message_id=msg.message_id,
                        reply_markup=show_keyboard(name),
                    )
            else:
                logger.error("Unable to retrieve old OTP on refresh")
        else:
            logger.error("Called cbk_show with no message")



import json

from pulumi_aws import dynamodb, iam

tags = {"app": "yaotpbot", "env": "production"}

table = dynamodb.Table(
    "yaotpbot-table",
    name="yaotpbot-table",
    attributes=[
        dynamodb.TableAttributeArgs(
            name="id",
            type="S",
        ),
        dynamodb.TableAttributeArgs(
            name="name",
            type="S",
        ),
    ],
    billing_mode="PROVISIONED",
    hash_key="id",
    range_key="name",
    read_capacity=5,
    write_capacity=5,
    tags=tags,
)


user = iam.User("yaotpbot", name="yaotpbot", tags=tags)
ak = iam.AccessKey("yaotpbot-ak", user=user.name)
policy = iam.UserPolicy(
    "yaotpbot-dynamo-policy",
    name="yaotpbot-dynamo-policy",
    user=user.name,
    policy=table.arn.apply(
        lambda arn: json.dumps(
            {
                "Version": "2012-10-17",
                "Statement": [
                    {
                        "Sid": "Stmt1661462886736",
                        "Action": [
                            "dynamodb:DeleteItem",
                            "dynamodb:DescribeTable",
                            "dynamodb:GetItem",
                            "dynamodb:GetRecords",
                            "dynamodb:PutItem",
                            "dynamodb:Query",
                            "dynamodb:Scan",
                            "dynamodb:UpdateItem",
                            "dynamodb:UpdateTimeToLive",
                        ],
                        "Effect": "Allow",
                        "Resource": arn,
                    }
                ],
            }
        ),
    ),
)



from telegram import Update
from telegram.ext import CallbackContext

from ..helpers.keyboards import default_keyboard


def cmd_start(update: Update, _: CallbackContext) -> None:
    if msg := update.message:
        msg.reply_text("Choose somethind to do!", reply_markup=default_keyboard())



from typing import List, Tuple

from telegram import InlineKeyboardButton, InlineKeyboardMarkup

from ..constant import (
    BTN_ADD_LABEL,
    BTN_BACK_LABEL,
    BTN_LIST_LABEL,
    BTN_REFRESH_LABEL,
    BTN_REMOVE_LABEL,
    BTN_REMOVE_N_LABEL,
    BTN_REMOVE_Y_LABEL,
    CBK_DATA_ADD,
    CBK_DATA_HOME,
    CBK_DATA_LIST,
    CBK_DATA_REFRESH,
    CBK_DATA_REMOVE,
    CBK_DATA_REMOVE_LIST,
    CBK_DATA_RM_N,
    CBK_DATA_RM_Y,
    CBK_DATA_SHOW,
    LIST_ROW_LENGTH,
)
from . import persistence


def default_keyboard() -> InlineKeyboardMarkup:
    """
    Default keyboard.
    """
    return _keyboard(
        [
            [
                (BTN_ADD_LABEL, CBK_DATA_ADD),
                (BTN_REMOVE_LABEL, CBK_DATA_REMOVE_LIST),
                (BTN_LIST_LABEL, CBK_DATA_LIST),
            ]
        ]
    )


def add_name_keyboard() -> InlineKeyboardMarkup:
    """
    Keyboard used when a new OTP name is excpected in input.
    """
    return _keyboard([[(BTN_BACK_LABEL, CBK_DATA_HOME)]])


def show_keyboard(name: str) -> InlineKeyboardMarkup:
    """
    Keyboard used when a single OTP value is shown.
    """
    return _keyboard(
        [
            [
                (BTN_REFRESH_LABEL, f"{CBK_DATA_REFRESH}__{name}"),
                (BTN_BACK_LABEL, CBK_DATA_LIST),
            ]
        ]
    )


def ls_keyboard(chat_id: str) -> InlineKeyboardMarkup:
    """
    Keyboard used to list OTPs.
    """
    return _ls_keyboard_helper(chat_id, CBK_DATA_SHOW)


def ls_rm_keyboard(chat_id: str) -> InlineKeyboardMarkup:
    """
    Keyboard used to remove OTPs.
    """
    return _ls_keyboard_helper(chat_id, CBK_DATA_REMOVE)


def rm_keyboard() -> InlineKeyboardMarkup:
    """
    Keyboard used to confirm the deletion of an OTP.
    """
    return _keyboard(
        [[(BTN_REMOVE_Y_LABEL, CBK_DATA_RM_Y), (BTN_REMOVE_N_LABEL, CBK_DATA_RM_N)]]
    )


def _keyboard(args: List[List[Tuple[str, str]]]) -> InlineKeyboardMarkup:
    """
    Helper function used to generate a keyboard with multiple buttons.

    Parameters
    ----------
    args : List[List[Tuple[str, str]]]
        A list of list describing the keyboard layout. Each list represent a row of the layout and
        each element is a tuple in form (str,str), where the first element is the label of the button
        and the second one if the prefix for the CallbackQuery associated with button.

    Returns
    -------
    InlineKeyboardMarkup
        _description_
    """
    return InlineKeyboardMarkup(
        [[InlineKeyboardButton(k[0], callback_data=k[1]) for k in j] for j in args]
    )


def _ls_keyboard_helper(chat_id: str, cbk_prefix: str) -> InlineKeyboardMarkup:
    """
    Helper function to generate keyboards who list existing seed.

    Parameters
    ----------
    chat_id : str
        Identifier of the chat
    cbk_prefix : str
        Prefix of the CallbackQuery to register on button click.

    Returns
    -------
    InlineKeyboardMarkup
        A keyboard with existing seeds as buttons.
    """
    otp_keys = [k[0] for k in persistence.load(chat_id)]
    keyboard_matrix = []
    keyboard_row: List[Tuple[str, str]] = []
    for i in range(len(otp_keys)):
        if i % (LIST_ROW_LENGTH) == 0 and i != 0:
            keyboard_matrix.append(keyboard_row)
            keyboard_row = []
        keyboard_row.append((otp_keys[i], f"{cbk_prefix}__{otp_keys[i]}"))
    keyboard_matrix.append(keyboard_row)

    keyboard_matrix.append([(BTN_BACK_LABEL, CBK_DATA_HOME)])
    return _keyboard(keyboard_matrix)



import logging
from binascii import Error

import pyotp
from telegram import Update
from telegram.ext import (
    CallbackContext,
    CallbackQueryHandler,
    CommandHandler,
    ConversationHandler,
    Filters,
    MessageHandler,
)

from ..constant import (
    CBK_DATA_ADD,
    CMD_CANCEL,
    ST_ADD_NAME,
    ST_ADD_OPT,
    USER_DATA_KEY_ADD_MSG_ID,
    USER_DATA_KEY_ADD_NAME,
)
from ..handlers.cancel import cbk_cancel
from ..helpers import keyboards, persistence

logger = logging.getLogger(__name__)


def cbk_add(update: Update, context: CallbackContext) -> int:
    if cbk := update.callback_query:
        context.bot.answer_callback_query(cbk.id)
        if msg := update.callback_query.message:
            context.bot.send_message(
                chat_id=msg.chat.id,
                text="Let's insert a new one. How should we call it? 💬\n"
                "Run /cancel command to abort.",
            )
            return ST_ADD_NAME
        else:
            logger.error("Called cbk_add with no message")
    return -1


def cbk_on_name(update: Update, context: CallbackContext) -> int:
    if (oldmsg := update.message) and ((userdata := context.user_data) is not None):
        userdata[USER_DATA_KEY_ADD_NAME] = update.message.text
        msg = oldmsg.reply_text(f"Great! Now insert the OTP seed for {oldmsg.text} 🎲🎲")
        userdata[USER_DATA_KEY_ADD_MSG_ID] = msg.message_id
        return ST_ADD_OPT
    logger.error("Called cbk_on_name with no message")
    return -1


def cbk_on_otp(update: Update, context: CallbackContext) -> int:

    if (
        (msg := update.message)
        and (txt := msg.text)
        and ((userdata := context.user_data) is not None)
    ):
        try:
            otp = pyotp.TOTP(txt)
            otp.now()
        except Error:
            name = userdata[USER_DATA_KEY_ADD_NAME]
            msg = msg.reply_text(
                f"😨😨 Oooops! Seems like this seed is invalid.\nInsert again the seed for {name} 🎲🎲"
            )
            userdata[USER_DATA_KEY_ADD_MSG_ID] = msg.message_id
            return ST_ADD_OPT

        persistence.put(
            str(msg.chat.id),
            userdata[USER_DATA_KEY_ADD_NAME],
            str(txt),
        )
        context.bot.delete_message(
            chat_id=msg.chat.id,
            message_id=userdata[USER_DATA_KEY_ADD_MSG_ID],
        )
        msg.delete()
        msg.reply_text(
            "🥳🥳 OTP successfully added.. I hid the seed for you 🤐\n" "What now?",
            reply_markup=keyboards.default_keyboard(),
        )
        return ConversationHandler.END

    logger.error(
        "Called cbk_on_otp with no message or no text on message or no user data."
    )
    return -1


def cnvs_hdl_add() -> ConversationHandler:
    return ConversationHandler(
        entry_points=[CallbackQueryHandler(cbk_add, pattern=CBK_DATA_ADD)],
        states={
            ST_ADD_NAME: [MessageHandler(Filters.text & ~Filters.command, cbk_on_name)],
            ST_ADD_OPT: [MessageHandler(Filters.text & ~Filters.command, cbk_on_otp)],
        },
        fallbacks=[CommandHandler(CMD_CANCEL, cbk_cancel)],
    )



from telegram import Update
from telegram.ext import CallbackContext


def cmd_help(update: Update, _: CallbackContext) -> None:
    if msg := update.message:
        msg.reply_text("You can start the bot typing /start.")
